# installation Se4-AD
# inspiré du install_phase2 - franck.molle@sambaedu.org
# 
# fichier à sourcer dans les postinst, donc pas d'interactivité



# ---------- Début des fonctions ----------#

# Fonction provision d'un nouvel AD - cas new installation

function provision_new_ad()
{
echo "Lancement d'une nouvelle installation de Samba AD avec sambatool" 
samba-tool domain provision --realm=$config_REALM --domain=$config_DOMAIN --adminpass=$config_ldap_admin_passwd
}
function write_krb5()
{
    	# renseignement des parametres indispensables
        # kerberos
        cp -a /usr/share/doc/sambaedu/krb5.conf.example /etc/krb5.conf
        write_param /etc/krb5.conf config_REALM
}
function write_smb()
{
        # samba
        cp -a /usr/share/doc/sambaedu/smb_ad-dc.conf.example /etc/samba/smb.conf
        write_param /etc/samba/smb.conf config_REALM
        write_param /etc/samba/smb.conf config_DOMAIN
        write_param /etc/samba/smb.conf config_nameserver
        
}

# Fonction activation samba ad-dc
function activate_smb_ad()
{
systemctl stop samba || true
systemctl stop smbd || true
systemctl stop nmbd || true
systemctl stop winbind  || true
systemctl disable samba || true
systemctl disable winbind || true
systemctl disable nmbd || true
systemctl disable smbd || true

systemctl unmask samba-ad-dc
systemctl enable samba-ad-dc
systemctl mask samba winbind nmbd smbd || true

systemctl start samba-ad-dc
}


# Fonction permettant l'ajout d'une OU ds l'annuaire AD
function ldbadd_ou()
{
local dn_add=$1
local rdn_add=$2
local desc_add=$3
ldbmodify -v -H /var/lib/samba/private/sam.ldb <<EOF
dn: $dn_add
changetype: add
objectClass: organizationalUnit
objectClass: top
instanceType: 4
OU: $rdn_add
description: $desc_add
EOF
}

# Fonction permettant le déplacement d'un groupe ou un utilisateur ds l'annuaire AD
function ldbmv_grp()
{
local dn_mv=$1
local rdn_mv=$2
local target_dn_mv=$3

ldbmodify -v -H /var/lib/samba/private/sam.ldb <<EOF
dn: $dn_mv
changetype: moddn
newrdn: $rdn_mv
deleteoldrdn: 1
newsuperior: $target_dn_mv
EOF
}

# Fonction permettant l'ajout des parties spécifiques à SE4 avec récup des données de l'ancien SE3 ds l'annuaire AD
function modif_ldb()
{
echo "Ajout des branches de l'annuaire propres à SE4"
ldbadd_ou "$config_groups_rdn,$config_ldap_base_dn" "Groups" "Branche des groupes"
ldbadd_ou "$config_cours_rdn,$config_groups_rdn,$config_ldap_base_dn" "Cours" "Branche des groupes cours"
ldbadd_ou "$config_matieres_rdn,$config_groups_rdn,$config_ldap_base_dn" "Matieres" "Branche des groupes matiere"
ldbadd_ou "$config_classes_rdn,$config_groups_rdn,$config_ldap_base_dn" "Classes" "Branche des groupes classe"
ldbadd_ou "$config_equipes,$config_groups_rdn,$config_ldap_base_dn" "Equipes" "Branche des groupes cours"
ldbadd_ou "$config_other_groups_rdn,$config_groups_rdn,$config_ldap_base_dn" "Autres" "Branche des groupes divers"
ldbadd_ou "$config_projets_rdn,$config_groups_rdn,$config_ldap_base_dn" "Projets" "Branche des groupes de projets"

ldbadd_ou "$config_people_rdn,$config_ldap_base_dn" "Utilisateurs" "Branche des utilisateurs SambaEdu"
ldbadd_ou "OU=Administratifs,$config_people_rdn,$config_ldap_base_dn" "Administratifs" "Branche des administratifs"
ldbadd_ou "OU=Profs,$config_people_rdn,$config_ldap_base_dn" "Profs" "Branche des profs"
ldbadd_ou "OU=Eleves,$config_people_rdn,$config_ldap_base_dn" "Eleves" "Branche des eleves"
ldbadd_ou "$config_trash_rdn,$config_ldap_base_dn" "Trash" "Branche de la corbeille"
ldbadd_ou "$config_parcs_rdn,$config_ldap_base_dn" "Parcs" "Branche des groupes parcs"
ldbadd_ou "$config_rights_rdn,$config_ldap_base_dn" "Rights" "Branche des droits"
ldbadd_ou "$config_equipements_rdn,$config_ldap_base_dn" "Equipements" "Branche équipements réseau, imprimantes"
ldbadd_ou "$config_delegations_rdn,$config_ldap_base_dn" "Delegations" "Branche Delegations"
}
# Fonction permettant la mise à l'heure du serveur 
function set_time()
{
echo "Configuration de Ntp et mise à l'heure"
chown -R root:ntp /var/lib/samba/ntp_signd
if [ -n "$config_ip_amon" ]; then
    echo "on utilise l'adresse de l'amon $config_ip_amon comme serveur de temps local"
    sed -i "s/^#server.*$/server $config_ip_amon/"  /etc/ntp.conf || true
fi
grep -q "ntpsigndsocket" /etc/ntp.conf ||
echo "# configuration specifique sambaedu
ntpsigndsocket /var/lib/samba/ntp_signd
restrict default mssntp
# Synchronize on myself (127.127 = myself, 1 = local clock)
server 127.127.1.0
# Stratum 10 (arbitrary)
fudge 127.127.1.0 stratum 10
tinker panic 0
" >> /etc/ntp.conf

systemctl restart ntp || true
systemctl enable ntp || true
}

# Fonction permettant l'écriture de resolv.conf car AD est DNS du domaine
function write_resolvconf()
{
cat >/etc/resolv.conf<<END
search $config_domain
nameserver 127.0.0.1
END
}


# Fonction permettant de se connecter ssh root sur se4-AD et avec la clé sambaedu
function Permit_ssh_by_password()
{
grep -q "^PermitRootLogin yes" /etc/ssh/sshd_config || echo "PermitRootLogin yes" >> /etc/ssh/sshd_config
/usr/sbin/service ssh restart
}

# Fonction permettant de descendre le niveau de complexité des pass utilisateurs
function change_policy_passwords() {
echo -e "Assouplissement de la politique des mots de passe pour les autres comptes"
samba-tool domain passwordsettings set --complexity=off
samba-tool domain passwordsettings set --history-length=0
samba-tool domain passwordsettings set --min-pwd-age=0
samba-tool domain passwordsettings set --max-pwd-age=0
}

function create_www-sambaedu()
{
samba-tool user create www-sambaedu --description="Utilisateur admin de l'interface web" --random-password 
samba-tool group addmembers "Domain Admins" www-sambaedu
samba-tool domain exportkeytab --principal=www-sambaedu@$config_REALM /etc/sambaedu/www-sambaedu.keytab
}

function create_admin_account()
{
echo -e "Création du compte admin du domaine sambaEdu 4"
samba-tool user create admin $config_admin_passwd --description="Admin du domaine sambaEdu" --userou="OU=Utilisateurs"
samba-tool group addmembers "Domain Admins" admin
}

# generation du contenu de la branche Rights 
function peuple_rights()
{
for cn_rights in Annu_is_admin se3_is_admin sovajon_is_admin printers_is_admin echange_can_administrate annu_can_read parc_can_view parc_can_manage no_Trash_user parc_can_clone 
do
samba-tool group add $cn_rights --groupou=$config_rights_rdn
samba-tool group addmembers $cn_rights Administrator
samba-tool group addmembers $cn_rights admin
done
}


function disable_ipv6()
{
if ! grep -q "#disable_ipv6" /etc/sysctl.conf; then
echo "#disable_ipv6
# désactivation de ipv6 pour toutes les interfaces
net.ipv6.conf.all.disable_ipv6 = 1

# désactivation de l’auto configuration pour toutes les interfaces
net.ipv6.conf.all.autoconf = 0

# désactivation de ipv6 pour les nouvelles interfaces (ex:si ajout de carte réseau)
net.ipv6.conf.default.disable_ipv6 = 1

# désactivation de l’auto configuration pour les nouvelles interfaces
net.ipv6.conf.default.autoconf = 0
" >> /etc/sysctl.conf
sysctl -p
fi
}


function check_ad()
{
	set_config sambaedu ldap_url "ldaps://${config_se4ad_name}.${config_domain}"

	ldapsearch  -xLLL -D "cn=$config_ldap_admin_name,$config_admin_rdn,$config_ldap_base_dn" \
            -b $config_ldap_base_dn -w "$config_ldap_admin_passwd" \
            "(&(objectClass=person)(samaccountname=$config_ldap_admin_name))"\
         -H $config_ldap_url
    return $?
}

# Fonction check samba ad-dc
function check_smb_ad()
{
echo -e "Attente de la disponibilité du service samba 4"
echo -e "L'initialisation de samba4 peut s'avérer assez longue lors de son tout premier lancement, jusqu'à quelques minutes"
echo -e "--> On attend que le service soit près avec une série de tests de connexion : smbclient -L localhost -U%"
echo -e "Pause de 60s pour commencer"
sleep 60
cpt_fin=12
for ((cpt=1; cpt <= cpt_fin ; cpt++))
do
	wt=20
	echo "Test de connexion $cpt"
	smbclient -L localhost -U% >$tempfile
		if [ "$?" != "0" ]; then
			echo "le service n'est pas encore prêt - nouvelle attente de $wt s"
			sleep $wt
		else
			cat $tempfile
			echo "le service est désormais fonctionnel, on peut passer à la suite !"
                        break
		fi
done
if [ "$cpt" = 13 ]; then
    echo -e "Aie ! - Connexion impossible sur l'AD"
fi
}

# jonction d'un dc à un domaine existant
function join_ad()
{
samba-tool domain join $config_domain DC -U"${config_ldap_admin_name}" --realm=${config_samba_DOMAIN} \
-W $config_samba_domain --dns-backend=SAMBA_INTERNAL \
--option="dns forwarder=${my_dnsserver}" --password="$config_ldap_passwd"
}

function rsync_sysvol()
{
rsync -e -e "ssh -i /etc/sambaedu/id_rsa" -aP root@${config_se4ad_name}:/var/lib/samba/sysvol/ /var/lib/samba/sysvol/
}
